#include <stdio.h>
#include <math.h>
#include <ctype.h>

/*
 *      Convert geographic coordinates to utm.
 * Method used is Clarke 1866 spheriod, courtesy Des Shaw at AeroGeo.
 */

#define A  6367399.6893
#define B    16216.9438
#define C       17.2089
#define D        0.0225
#define A1_CLARKE 6378206.4
#define B1_CLARKE 6356583.8
#define A1_NAD83  6378137.0000
#define B1_NAD83  6356752.3142
#define SC     123.0

main(argc, argv)
int argc;
char *argv[];
{
        
        double e1, e2, cm, cr, fn;
        double dla, mla, sla, dlo, mlo, slo;
        double latr, lonr;
        double t, et2, i, n, ma;
        double xutm, yutm;
        double a1, b1;

        int argp;

        char ns, ew, *tc, spheroid[255];

        int p;

        cm = SC;
        ns = 'n';
        ew = 'w';
        a1 = A1_CLARKE;
        b1 = B1_CLARKE;
        strcpy(spheroid, "(Clarke 1866)");
        for (argp = 1; argp < argc; argp++) {
                if (isalpha(*argv[argp])) {
                        for (tc = argv[argp]; *tc != '\0'; tc ++) {
                                switch (*tc) {
                                case 'n':
                                        ns = 'n';
                                        break;
                                case 's':
                                        ns = 's';
                                        break;
                                case 'e':
                                        ew = 'e';
                                        break;
                                case 'w':
                                        ew = 'w';
                                        break;
                                case 'N':
                                        a1 = A1_NAD83;
                                        b1 = B1_NAD83;
                                        strcpy(spheroid, "(NAD 1983)");
                                        break;
                                default:
                                        fprintf(stderr, "usage: gtoutm [s] [n] [e] [w] [central-meridian]\n");
                                        exit(1);
                                }
                        }
                } else if (isdigit(*argv[argp])) {
                        if (sscanf(argv[argp], "%lg", &cm) != 1) {
                                fprintf(stderr, "usage: gtoutm [s] [n] [e] [w] [central-meridian]\n");
                                exit(1);
                        }
                } else {
                        fprintf(stderr, "usage: gtoutm [s] [n] [e] [w] [central-meridian]\n");
                        exit(1);
                }
        }
        cr = cm * M_PI / 180.0;

        printf("Geographics to utm conversion %s.\n", spheroid);
        if (ns == 'n')
                fn = 0.0;
        else
                fn = 10000000.0;
        printf("Origin: latitude %1.2f %c; longitude %1.2lf %c.\n", 0.0, ns, cm, ew);
        printf("Scale factor at central meridian = 0.9996\n");
        printf("False coordinates: north %1.2lf m; east 500000.00 m.\n\n", fn);
        printf("Points:\n\n");
        printf(" %18s   %14s  %15s %15s\n", "latitude", "longitude", "eastings", 
                "northings");
        printf("%3s %3s %2s %7s   %3s %2s %7s   %15s %15s\n", "#", "d", "m", "s",
                "d", "m", "s", "meters", "meters");

        e1 = 1.0 - b1*b1/(a1*a1);
        e2 = a1*a1/(b1*b1) - 1.0;
        printf("og2u: a1=%.6lf b1=%.6lf e1=%.6lf e2=%.6lf\n",a1,b1,e1,e2);
        while (scanf("%d %lg %lg %lg %lg %lg %lg", &p, &dla, &mla, &sla, &dlo, &mlo, &slo) == 7) {
                latr = (dla + mla/60.0 + sla/3600.0) * M_PI / 180.0;    
                lonr = (dlo + mlo/60.0 + slo/3600.0) * M_PI / 180.0;
                t = tan(latr);
                printf("og2u: dla=%.8lf mla=%.8lf sla=%.8lf\n",dla,mla,sla);
                printf("og2u: dlo=%.8lf mlo=%.8lf slo=%.8lf\n",dlo,mlo,slo);
                printf("og2u: lat=%.8lf lon=%.8lf t=%.8lf\n",latr,lonr,t);
                et2 = e2 * pow(cos(latr), 2.0);
                i = cr - lonr;
                n = a1 / sqrt(1.0 - e1*pow(sin(latr), 2.0));
                ma = A*latr - B*sin(2.0*latr) + C*sin(4.0*latr) - D*sin(6.0*latr);
                printf("og2u: et2=%.8lf i=%.8lf n=%.8lf ma=%.8lf\n",
                       et2,i,n,ma);
                xutm = 0.9996*(i*n*cos(latr)*(1.0 + i*i/6.0*pow(cos(latr), 2.0)*
                        (1.0 - t*t + et2) + pow(i, 4.0)/120.0*pow(cos(latr), 4.0)*(5.0 +
                        14.0*et2 - 18.0*t*t - 58.0*et2 + pow(t, 4.0))));
                printf("og2u: xutm=%.4lf\n",xutm);
                if (ew == 'w')
                        xutm += 500000.0;
                else
                        xutm = 500000.0 - xutm;
                yutm = 0.9996*(ma + i*i/2.0*n*sin(latr)*cos(latr)*(
                        1.0 + i*i/12.0*pow(cos(latr), 2.0)*(5.0 - t*t + 9.0*et2 +4.0*et2*et2) +
                        pow(i, 4.0)/360.0*pow(cos(latr), 4.0)*(61.0 + 270.0*et2 -
                        58.0*t*t + pow(t, 4.0) - 330.0*t*t*et2)));
                printf("og2u: yutm=%.4lf\n",yutm);
                if (ns == 's')
                        yutm = 10000000.0 - yutm;
                printf("%3d %3.0lf %2.0lf %7.4lf %c %3.0lf %2.0lf %7.4lf %c %15.3lf %15.3lf\n",
                        p, dla, mla, sla, ns, dlo, mlo, slo, ew, xutm, yutm);
        }
}
