#include <stdio.h>
#include <math.h>
#include <ctype.h>

/*
 *	Convert utm coordinates to geographic.
 * Method used is Clarke 1866 spheriod, courtesy Des Shaw at AeroGeo.
 */

#define A  6367399.6893
#define B    16216.9438
#define C       17.2089
#define D        0.0225
#define A1_CLARKE 6378206.4
#define B1_CLARKE 6356583.8
#define A1_NAD83  6378137.0000
#define B1_NAD83  6356752.3142
#define SC     123.0
#define EPS      1.0e-14

main(argc, argv)
int argc;
char *argv[];
{
	
	double e1, e2, fn, cm;
	double dla, mla, sla, dlo, mlo, slo;
	double lat, lon;
	double t, et2, i, f1, fl, x, y, nn;
	double xutm, yutm;
	double a1, b1;

	int argp;

	char ns, ew, *tc, spheroid[255];

	int p;

	ns = 'n';
	ew = 'w';
	cm = SC;
	a1 = A1_CLARKE;
	b1 = B1_CLARKE;
	strcpy(spheroid, "(Clarke 1866)");

	for (argp = 1; argp < argc; argp++) {
		if (isalpha(*argv[argp])) {
			for (tc = argv[argp]; *tc != '\0'; tc ++) {
				switch (*tc) {
				case 'n':
					ns = 'n';
					break;
				case 's':
					ns = 's';
					break;
				case 'e':
					ew = 'e';
					break;
				case 'w':
					ew = 'w';
					break;
				case 'N':
					a1 = A1_NAD83;
					b1 = B1_NAD83;
					strcpy(spheroid, "(NAD 1983)");
					break;
				default:
					fprintf(stderr, "usage: utmtog [s] [n] [e] [w] [central-meridian]\n");
					exit(1);
				}
			}
		} else if (isdigit(*argv[argp])) {
			if (sscanf(argv[argp], "%lg", &cm) != 1) {
				fprintf(stderr, "usage: utmtog [s] [n] [e] [w] [central-meridian]\n");
				exit(1);
			}
		} else {
			fprintf(stderr, "usage: utmtog [s] [n] [e] [w] [central-meridian]\n");
			exit(1);
		}
	}

	printf("Utm to geographics conversion %s.\n", spheroid);
	if (ns == 'n')
		fn = 0.0;
	else
		fn = 10000000.0;
	printf("Origin: latitude %1.2lf %c; longitude %1.2lf %c.\n", 0.0, ns, cm, ew);
	printf("Scale factor at central meridian = 0.9996\n");
	printf("False coordinates: north %1.2lf m; east 500000.00 m.\n\n", fn);
	printf("Points:\n\n");
	printf(" %18s   %14s  %15s %15s\n", "latitude", "longitude", "eastings", 
		"northings");
	printf("%3s %3s %2s %7s   %3s %2s %7s   %15s %15s\n", "#", "d", "m", "s",
		"d", "m", "s", "meters", "meters");

	e1 = 1.0 - b1*b1/(a1*a1);
	e2 = a1*a1/(b1*b1) - 1.0;
	while (scanf("%d %lg %lg", &p, &xutm, &yutm) == 3) {
		if (ew == 'w')
			x = (xutm - 500000.0) / 0.9996;
		else
			x = (500000.0 - xutm) / 0.9996;
		if (ns == 's')
			y = (10000000.0 - yutm) / 0.9996;
		else
			y = yutm / 0.9996;
		for (f1 = y / A; fabs ((fl = (y +B*sin(2.0*f1) - C*sin(4.0*f1) + D*sin(6.0*f1)) / A) - f1) > EPS; f1 = fl) {}
		nn = a1 / sqrt(1.0 - e1*pow(sin(fl), 2.0));
		t = tan(fl);
		et2 = e2 * pow(cos(fl), 2.0);
		i = fl + (x*x*(-((1.0+et2)*t/(2.0*nn*nn)))+(
			pow(x, 4.0) * (t/(24.0*pow(nn, 4.0))*(5.0 + 3.0*t*t + 6.0*et2 -
			6.0*t*t*et2)) + (pow(x, 6.0)*t/720.0/pow(nn, 5.0)/nn*(61.0 +
			90.0*t*t + 46.0*et2 + 54*pow(t, 4.0) - 252.0*et2*t*t - 3.0*et2*et2))));
		lat = i*180.0/M_PI;
		i = x*(1.0/(nn*cos(fl))) + pow(x, 3.0)*(-((1.0+2.0*t*t+et2)/
			(6.0*pow(nn, 3.0)*cos(fl)))) + pow(x, 5.0)*(1.0/(120*pow(nn, 5.0)*
			cos(fl))*(5.0+28.0*t*t+24.0*pow(t, 4.0)+6.0*et2+8.0*t*t*et2));
		lon = cm - i*180.0/M_PI;
		dla = floor(lat);
		mla = floor((lat - dla)*60.0);
		sla = ((lat - dla)*60.0 - mla)*60.0;
		dlo = floor(lon);
		mlo = floor((lon - dlo)*60.0);
		slo = ((lon - dlo)*60.0 - mlo)*60.0;
		printf("%3d %3.0lf %2.0lf %7.4lf %c %3.0lf %2.0lf %7.4lf %c %15.3lf %15.3lf\n",
			p, dla, mla, sla, ns, dlo, mlo, slo, ew, xutm, yutm);
	}
}
