      SUBROUTINE TMPLXY(PHI,ELAM,A,B,SF,XO,YO,CMRAD,X,Y)
C
C         NAME:       TMPLXY
C
C         TYPE:       SUBROUTINE
C
C         FUNCTION:   COMPUTES THE X,Y COORDINATES FOR THE TRANSVERSE
C                     MERCATOR PROJECTION GIVEN THE GEOGRAPHIC
C                     COORDINATES (LATITUDE & LONGITUDE)  - THOMAS (1952
C
C         VERSION:    VERSION 1 (IBM)
C
C         ANALYST:    R.R.STEEVES  (MAY 1977)
C
C         MODIFIED:   K.W.SCHUURMAN   (JANUARY 1981)
C
C         STATEMENT:  CALL TMPLXY(PHI,ELAM,A,B,SF,XO,YO,CMRAD,X,Y)
C
C
C                     NAME   I/O DESCRIPTION
C                     ----   --- -----------
C
C                     PHI     I  LATITUDE IN RADIANS (POSITIVE NORTH)
C                     ELAM    I  LONGITUDE IN RADIANS (POSITIVE EAST)
C                     A       I  SEMI-MAJOR AXIS OF REFERENCE ELLIPSOID
C                     B       I  SEMI-MINOR AXIS OF REFERENCE ELLIPSOID
C                     SF      I  SCALE OF THE CENTRAL MERIDIAN
C                     XO      I  FALSE EASTING OF THE CENTRAL MERIDIAN
C                     YO      I  FALSE NORTHING
C                     CMRAD   I  LONGITUDE OF THE CENTRAL MERIDIAN (RADI
C                     X       O  TRANSVERSE MERCATOR EASTING
C                     Y       O  TRANSVERSE MERCATOR NORTHING
C
C         LANGUAGE:   FORTRAN G1
C
C         EXTERNALS:  MERARC(PHI, A, B, SPHI)
C
C
      IMPLICIT REAL*8(A-H,O-Z)
      DLAM=ELAM-CMRAD
      SP=DSIN(PHI)
      CP=DCOS(PHI)
      T=DTAN(PHI)
      E=DSQRT((A**2-B**2)/A**2)
      ETA=DSQRT((A**2-B**2)/B**2*CP**2)
      CALL MERARC(PHI,A,B,SPHI)
      DN=A/DSQRT(1.0D0-E**2*SP**2)
      X=DN*(DLAM*CP+DLAM**3*CP**3/6.0D0*(1.0D0-T**2+ETA**2)+DLAM**5*CP
     1   **5/120.0D0*(5.0D0-18.0D0*T**2+T**4+14.0D0*ETA**2-58.0D0*T**2*
     2   ETA**2+13.D0*ETA**4+4.D0*ETA**6-64.D0*ETA**4*T**2-24.D0*ETA**6
     3   *T**2) +DLAM**7/5040.0D0*CP**7*(61.0D0-479.0D0*T**2+179.0D0*
     4   T**4-T**6))
      Y=SPHI+DN*   (DLAM**2/2.0D0*SP*CP+DLAM**4/24.0D0*SP*CP**3*(5.0D0-
     1   T**2+9.0D0*ETA**2+4.0D0*ETA**4)+DLAM**6/720.0D0*SP*CP**5*
     2   (61.0D0-58.0D0*T**2+T**4+270.0D0*ETA**2-330.0D0*T**2*ETA**2
     3   +445.D0*ETA**4+324.D0*ETA**6-680.D0*ETA**4*T**2+88.D0*ETA**8
     4   -600.D0*ETA**6*T**2-192.D0*ETA**8*T**2)+DLAM**8/40320.D0*SP*CP*
     5   *7*(1385.D0-3111.D0*T**2+543.D0*T**4-T**6))
      X=SF*X+XO
      Y=SF*Y+YO
      RETURN
      END
