
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <tfictypes.h>

char *prog_name;
bool debug;

/*
 *      Convert geographic coordinates to utm.
 * Method used is Clarke 1866 spheriod, courtesy Des Shaw at AeroGeo.
 * 1990 August 13 - bug in analysing options (-C treated as -N) fixed.
 */

#define A1_CLARKE 6378206.4
#define B1_CLARKE 6356583.8
#define A1_NAD83  6378137.0000
#define B1_NAD83  6356752.3142
#define SC     123.0

double a1, b1;
double cm,fn,fe,sf;
bool verbose;
char ns,ew,spheroid[255];

void usage()
  {fprintf(stderr,"usage: %s [-newsCN[m cm]vD]\n",prog_name);
   exit(1);}

   /* globals for getopt() */

int optind, opterr;
char *optarg;

void args(argc,argv)
      int argc;
      char *argv[];
  {char c;
   extern int getopt();
   extern double atof();
   cm = SC;
   ns = 'n';
   ew = 'w';
   a1 = A1_CLARKE;
   b1 = B1_CLARKE;
   strcpy(spheroid, "(Clarke 1866)");
   verbose = FALSE;
   optind = 1;  opterr = 0;
   while ((c = getopt(argc,argv,"newsCNm:vD")) != EOF)
     {switch (c)
        {case 'n':
            ns = 'n';
            break;
         case 's':
            ns = 's';
            break;
         case 'e':
            ew = 'e';
            break;
         case 'w':
            ew = 'w';
            break;
         case 'v':
            verbose = TRUE;
            break;
         case 'C':
            a1 = A1_CLARKE;
            b1 = B1_CLARKE;
            strcpy(spheroid, "(Clarke 1866)");
            break;
         case 'N':
            a1 = A1_NAD83;
            b1 = B1_NAD83;
            strcpy(spheroid, "(NAD 1983)");
            break;
         case 'm': /* set the central meridian */
            cm = atof(optarg);
            break;
         case 'D':
            debug = TRUE;
            break;
         case '?':
            usage();}}
   if (ns == 'n')
      fn = 0.0;
   else
      fn = 10000000.0;
   fe = 500000.0;
   sf = 0.9996;}

void show()
  {printf("Geographics to utm conversion %s.\n",spheroid);
   printf("Origin: latitude %1.2f %c; longitude %1.2lf %c.\n",
          0.0,ns,cm,ew);
   printf("Scale factor at central meridian = %6.4lf\n",sf);
   printf("False coordinates: north %1.2lf m; east %1.2lf m.\n\n",fn,fe);
   printf("Points:\n\n");
   printf(" %18s   %14s  %15s %15s\n","latitude","longitude","eastings", 
          "northings");
   printf("%3s %3s %2s %7s   %3s %2s %7s   %15s %15s\n","#","d","m","s",
          "d","m","s","meters","meters");}

void main(argc,argv)
      int argc;
      char *argv[];
  {/* geographics to UTMs */
   int p,idla,imla,idlo,imlo;
   double sla,slo,cr,xutm,yutm;
   double latr,lonr;
   double oxutm,oyutm;
   extern double dms_rad();
   extern void rad_utm();
   extern void radoutm();
   prog_name = argv[0];
   debug = FALSE;
   args(argc,argv);
   cr = cm * M_PI / 180.0;
   if (verbose)
     {show();}
   while (scanf("%d",&p) == 1 &&
          (p <= 0 ||
           scanf("%d%d%lg%d%d%lg",&idla,&imla,&sla,&idlo,&imlo,&slo) == 6))
     {if (p < 0)
        {/* set new central meridian */
         cr = (cm = (double) -p) * M_PI / 180.0;
         if (verbose)
           {show();}}
      else
        {/* regular point */
         latr = dms_rad(idla,imla,sla);
         lonr = dms_rad(idlo,imlo,slo);
         rad_utm(latr,-lonr,a1,b1,sf,fe,fn,-cr,&xutm,&yutm);
         if (!verbose)
           {printf("%4d%16.3lf%16.3lf\n",p,xutm,yutm);}
         else
           {printf("%4d%4d%3d%8.4lf %c%4d%3d%8.4lf %c%16.3lf%16.3lf\n",
                   p,idla,imla,sla,ns,idlo,imlo,slo,ew,xutm,yutm);}}}}
