
   /* Mapsheet number -> geographic coordinates 
      Modifications:
         clh 90/11/11, wrong 1:10,000 numbering
         tbr 90/11/21, another numbering fix
         tbr 91/02/19, added 1:50,000 and 25 tics
         tbr 91/08/19, added Alberta style 1:20,000 & 1:10,000 names
                       and general 1:250,000 names */

#include <stdio.h>
#include <ctype.h>
#include <tfictypes.h>

char *prog_name;
int num_coords;

   /* Units west and north from the SE corner of a map, in order
      of generated TICS.  For 4, 9 or 25 TICS (TFIC convention):
            3 25  7 24  4
           23 22 21 20 19
            6 18  9 17  8
           16 15 14 13 12
            2 11  5 10  1
    */

                          /* SE  SW  NW  NE   S   W   N   E   C */
static hword mul_west[]  = {  0,  4,  4,  0,  2,  4,  2,  0,  2,
                              1,  3,  0,  1,  2,  3,  4,  1,  3,
                              0,  1,  2,  3,  4,  1,  3};
static hword mul_north[] = {  0,  0,  4,  4,  0,  2,  4,  2,  2,
                              0,  0,  1,  1,  1,  1,  1,  2,  2,
                              3,  3,  3,  3,  3,  4,  4};

#define SCALE_UNKNOWN -1
#define SCALE_250     0
#define SCALE_50      1
#define SCALE_20BC    2
#define SCALE_10BC    3
#define SCALE_20AB    4
#define SCALE_10AB    5

   /* hundredths of seconds between closest TICS indexed by SCALE_* */
static long int uwe[] = {180000, 45000, 18000,  9000, 22500, 11250};
static long int uns[] = { 90000, 22500,  9000,  4500, 11250,  5625};

static char *map_desc[] =
  {"DDDL          - 1:250,000",
   "DDDLdd        - 1:50,000 (dd is 1-origin index into 4x4 grid",
   "                          13 14 15 16",
   "                          12 11 10 09",
   "                          05 06 07 08",
   "                          04 03 02 01)",
   "DDDLddqq      - 1:20,000 - Alta (qq is {se,sw,nw,ne} quadrant of 1:50,000",
   "                                 quadrants:  nw=2  3=ne",
   "                                             sw=0  1=se ",
   "                                 (for compatibility with BC 1:10 scheme))",
   "DDDLddqq[.]qq - 1:10,000 - Alta (qq is {se,sw,nw,ne} quadrant of 1:20,000)",
   "DDDLddd       - 1:20,000 - B.C. (ddd is 1-origin index into 10x10 grid",
   "                                 091...100",
   "                                  :     :",
   "                                 001...010)",
   "DDDLddd[.]q   - 1:10,000 - B.C. (q is 1-origin index into 2x2 grid)",
   NULL};

static int mapscale(map,num,let,p1,p2,p3)
      char *map;
      long int *num,*let,*p1,*p2,*p3;
  {/* parse a map name */
   long int n,nd;
   char c;
   /* pick up first part + letter */
   for (n = 0;  isdigit(c = *map++);  )
     {n = n * 10 + (c - '0');}
   *num = n;
   if (islower(c))
     {c = toupper(c);}
   c -= 'A';  *let = c;  nd = 0;
   /* pick up digits */
   for (n = 0;  isdigit(c = *map++) && ++nd <= 3;  )
     {n = n * 10 + (c - '0');}
   if (nd == 0)
     {return(SCALE_250);}
   *p1 = n -= 1;  /* get rid of 1-origin */
   if (nd >= 3)
     {/* B.C. 1:20 or 1:10 */
      if (c == '.')
        {c = *map++;  nd += 1;}
      if (!(0 <= n && n <= 99) || (c != 0 && *map != 0))
        {return(SCALE_UNKNOWN);}
      if (nd == 3)
        {return(SCALE_20BC);}
      n = (c - '1');
      if (!(0 <= n && n <= 3))
        {return(SCALE_UNKNOWN);}
      *p2 = n;
      return(SCALE_10BC);}
   if (nd != 2 || !(0 <= n && n <= 15))
     {return(SCALE_UNKNOWN);}
   if (c == 0)
     {return(SCALE_50);}
   /* Alta 1:20 or 1:10 */
   if (islower(c))
     {c = toupper(c);}
   if (!(c == 'N' || c == 'S'))
     {return(SCALE_UNKNOWN);}
   n = (c == 'N') ? 2 : 0;
   if (islower(c = *map++))
     {c = toupper(c);}
   if (!(c == 'W' || c == 'E'))
     {return(SCALE_UNKNOWN);}
   *p2 = n += (c == 'W') ? 0 : 1;
   if ((c = *map++) == '.')
     {c = *map++;}
   if (c == 0)
     {return(SCALE_20AB);}
   if (islower(c))
     {c = toupper(c);}
   if (!(c == 'N' || c == 'S'))
     {return(SCALE_UNKNOWN);}
   n = (c == 'N') ? 2 : 0;
   if (islower(c = *map++))
     {c = toupper(c);}
   if (!(c == 'W' || c == 'E') || *map != 0)
     {return(SCALE_UNKNOWN);}
   *p3 = n += (c == 'W') ? 0 : 1;
   return(SCALE_10AB);}

void map2geo(map,fd)
      char *map;
      FILE *fd;
  {/* write the geographic coordinates of map to fd in
      the order se,sw,nw,ne (see above) as:
        -cm
        1 lat long
        2 lat long
        3 lat long
        4 lat long               */
   long int num,let,p1,p2,p3,scale;
   long int dx,mx,dy,my,sx,sy;
   long int dw,mw,sw100,dn,mn,sn100,c;
   double sw,sn;
   p1 = 0;  p2 = p3 = 1;
   if ((scale = mapscale(map,&num,&let,&p1,&p2,&p3)) == SCALE_UNKNOWN)
     {fprintf(stderr,"%s: Eh? what kind of map is `%s'?\n",prog_name,map);
      exit(1);}
   /* compute degrees for southeast corner of grid from first part + letter */
   dx = 48 + 2 * (4 * (num/10) + ((((let>>2) & 0x1) * 0x3) ^ (let&0x3)));
   dy = 40 + 4 * (num%10) + (let>>2);
   /* write the negative of the UTM central meridian */
   fprintf(fd,"%d\n",-(3+6*(dx/6)));
   /* add trailing numbers */
   switch (scale)
     {case SCALE_250:
      case SCALE_50:
      case SCALE_20AB:
      case SCALE_10AB:
         sx = 450 * (((((p1 / 4) & 0x1) ? 3 - (p1 % 4) : p1 % 4) << 2) |
                     ((0x1 & (p2 + 1)) << 1) | (0x1 & (p3 + 1)));
         sy = 225 * ((p1 & 0xC) | (p2 & 0x2) | (p3 >> 1));
      /* printf("sy = 225*(a1=%d+a2=%d+a3=%d) = %d\n",p1&0xC,p2&0x2,p3>>1,sy);*/
         break;
      case SCALE_20BC:
      case SCALE_10BC:
         sx = 360 * (((9 - (p1 % 10)) << 1) | (0x1 & (p2 + 1)));
         sy = 180 * (((p1 / 10) << 1) | (p2 >> 1));}
   mx = sx / 60;  sx %= 60;
   my = sy / 60;  sy %= 60;
/* printf("my=%d sy=%d\n",my,sy);*/
   dx += mx / 60;  mx %= 60;
   dy += my / 60;  my %= 60;
   /* compute coordinates */
   for (c = 0;  c < num_coords;  c++)
     {sn100 = mul_north[c] * uns[scale] + sy * 100;
      mn = my + sn100 / 6000;  sn100 %= 6000;
      dn = dy + mn / 60;  mn %= 60;
      sn = sn100 / 100.0;
      sw100 = mul_west[c] * uwe[scale] + sx * 100;;
      mw = mx + sw100 / 6000;  sw100 %= 6000;
      dw = dx + mw / 60;  mw %= 60;
      sw = sw100 / 100.0;
      fprintf(fd,"%d   %2d %2d %5.2f   %2d %2d %5.2f\n",
              c+1,dn,mn,sn,dw,mw,sw);}}

   /* globals for getopt() */

int optind, opterr;
char *optarg;
extern int getopt();

void usage()
  {int n;
   fprintf(stderr,"usage: %s [-492] map ...\n",prog_name);
   fprintf(stderr,"   where each `map' is one of:\n");
   for (n = 0;  map_desc[n] != NULL;  n++)
     {fprintf(stderr,"   %s\n",map_desc[n]);}
   exit(1);}

void main(argc,argv)
      int argc;
      char *argv[];
  {/* convert args to geo coords */
   int m,c;
   prog_name = argv[0];
   num_coords = 4;
   optind = 1;  opterr = 0;
   while ((c = getopt(argc,argv,"4925")) != EOF)
     {switch (c)
        {case '9':
            num_coords = 9;
            break;
         case '4':
            num_coords = 4;
            break;
         case '2':
         case '5':
            num_coords = 25;
            break;
         case '?':
            usage();}}
   for (m = optind;  m < argc;  )
     {map2geo(argv[m++],stdout);}}
