
#include <stdio.h>
#include <math.h>
#include <tfictypes.h>

extern bool debug;

void utm_rad(x,y,a,b,sf,xo,yo,cmrad,phi,olam)
      double x;      /* UTM easting */
      double y;      /* UTM northing */
      double a;      /* semi-major axes of the reference ellipsoid */
      double b;      /* semi-minor axes of the reference ellipsoid */
      double sf;     /* scale of the central meridian */
      double xo;     /* false easting of the central meridian */
      double yo;     /* false northing of the central meridian */
      double cmrad;  /* the central meridian, in radians */
      double *phi;   /* latitude of the point in radians */
      double *olam;  /* longitude of the point in radians */
  {/* UTMs -> radians:
      Translated into C 23 Feb 89 by Tom Rushworth.
      (Much rearrangement of formulas done to eliminate exponentiation,
       the survival of the accuracy while the code was in the hands of the
       ignorant is unknown.)
      Original Fortran comments below:

         SUBROUTINE TMXYPL COMPUTES THE GEOGRAPHIC COORDINATES- LATITUDE
      AND LONGITUDE - GIVEN THE X,Y COORDINATES OF THE TRANSVERSE
      MERCATOR PROJECTION. THE EQUATIONS USED TO COMPUTE THE LONGITUDE
      AND LATITUDE ARE FROM THOMAS (1952). SUBROUTINE FPLAT IS USED
      TO COMPUTE THE FOOT-POINT LATITUDE.

      WRITTEN:  R.R. STEEVES, MAY, 1977

      Modified: Ken W. Schuurman ( To eliminate exponent overflow and
                                    underflow problems)
     */
   double e,phi1,sp,cp,eta,dn,dm,dlam,t;
   double a2,b2,e2,t2,sp2,cp2,eta2,dn2,x2,xdn,xdn2;
   double eta_term,t_term,esp;
   extern double fplat();
   if (debug)
     {fprintf(stderr,"utm_rad: x=%.3lf y=%.3lf xo=%.3lf yo=%.3lf\n",x,y,xo,yo);
      fprintf(stderr,"         a=%.6lf b=%.6lf\n",a,b);
      fprintf(stderr,"         sf=%.6lf cmr=%.15lf\n",sf,cmrad);}
   x  =  (x - xo) / sf;  x2 = x * x;
   y  =  (y - yo) / sf;
   a2 = a * a;  b2 = b * b;
   e2 = (a2 - b2) / a2;
   e = sqrt(e2);
   phi1 = fplat(a,b,y);
   if (debug)
     {fprintf(stderr,"utm_rad: phi1=%.15lf\n",phi1);}
   t = tan(phi1);  t2 = t * t;
   sp = sin(phi1);  sp2 = sp * sp;
   cp = cos(phi1);  cp2 = cp * cp;
   eta2 = cp2 * (a2 - b2) / b2;
   eta = sqrt(eta2);
   esp = 1.0 - e2*sp2;
   dn = a/sqrt(esp);  dn2 = dn * dn;
   xdn = x / dn;  xdn2 = xdn * xdn;
   dm = a*(1.0-e2)/sqrt(esp*esp*esp);
   eta_term = 61.0 + (90.0 + 45.0*t2) * t2
              + (46.0 + (-252.0 - 90.0*t2) * t2
                 + (-3.0 + (-66.0 + 225.0*t2) * t2
                    + (100.0 + 84.0*t2
                       + (88.0 - 192.0*t2)*eta2)*eta2)*eta2)*eta2;
   t_term = 1385.0 + (3633.0 + (4095.0 + 1575.0 * t2) * t2) * t2;
   *phi = phi1 + (t*x2/(2.0*dm*dn))
      * ((- 1.0
          + (x2/(12.0*dn2))
            * ((5.0 + 3.0*t2 + (1.0 - 4.0*eta2 - 9.0*t2)*eta2)
               - (x2/(30.0*dn2))*eta_term))
         + xdn2*xdn2*xdn2*t_term/20160.0);
   if (debug)
     {fprintf(stderr,"utm_rad: phi=%.15lg\n",*phi);}
   dlam = xdn
      * (1.0
         + (xdn2/6.0)
           * (-(1.0+2.0*t2+eta2)
              + (xdn2/20.0)
                *((5.0 + (28.0 +24.0*t2) * t2
                   + (6.0 + 8.0*t2
                      + (-3.0 + 4.0*t2
                         + (-4.0 + 24.0*t2)*eta2)*eta2)*eta2)
                  - (xdn2/42.0)
                    *(61.0 + (662.0 + (1320.0 +720.0 * t2) * t2) * t2))))/cp;
   if (debug)
     {fprintf(stderr,"utm_rad: dlam=%.15lg\n",dlam);}
   *olam  =  cmrad + dlam;}
